"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.Injector = void 0;

var _findIndex = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/find-index"));

var _splice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/splice"));

var _eventId = require("./eventId");

var semver = _interopRequireWildcard(require("semver"));

const EMIT_VERSION = "0.4.21";
const EVENT_NODE = {
  name: "__StatementCoverage",
  parameters: {
    parameters: [{
      name: "value",
      type: "VariableDeclaration",
      typeName: {
        name: "uint32",
        type: "ElementaryTypeName"
      }
    }],
    type: "ParameterList"
  },
  type: "EventDefinition"
};

class Injector {
  constructor(contract, solcVersion) {
    this.contract = contract;
    this.solcVersion = solcVersion;
    this.isEmitSupported = void 0;
    this.solcVersion = solcVersion;
    this.isEmitSupported = semver.gte(this.solcVersion, EMIT_VERSION);
  }

  process(injectionPoint) {
    switch (injectionPoint.type) {
      case "statement":
        return this.statement(injectionPoint);

      case "contractDefinition":
        return this.contractDefinition(injectionPoint);
    }
  }

  emitStatementNode(id) {
    // Depending on what solidity version we're handling, "emit" statements might not
    // be supported, so we check here.
    const functionCall = {
      arguments: [{
        number: id.toString(),
        type: "NumberLiteral",
        subdenomination: null
      }],
      expression: {
        name: "__StatementCoverage",
        type: "Identifier"
      },
      names: [],
      type: "FunctionCall"
    };

    if (this.isEmitSupported) {
      return {
        eventCall: functionCall,
        type: "EmitStatement"
      };
    }

    return {
      expression: functionCall,
      type: "ExpressionStatement"
    };
  }

  statement(injectionPoint) {
    const eventId = (0, _eventId.encrypt)(this.contract.id, injectionPoint.id); // Get the proper node that we're going to inject instead of building a string.
    // There _might_ be a better way to do this.

    const node = this.emitStatementNode(eventId);
    this.insertAt(node, injectionPoint);
  }

  contractDefinition(injectionPoint) {
    var _context;

    const contractNode = injectionPoint.node;
    (0, _splice.default)(_context = contractNode.subNodes).call(_context, 0, 0, EVENT_NODE);
  }

  insertAt(emit, injectionPoint) {
    var _context2, _context3;

    const {
      parent,
      node
    } = injectionPoint;

    if (!parent) {
      return;
    }

    const idx = (0, _findIndex.default)(_context2 = parent.statements).call(_context2, s => s.loc === node.loc);
    (0, _splice.default)(_context3 = parent.statements).call(_context3, idx, 0, emit);
  }

}

exports.Injector = Injector;
//# sourceMappingURL=injector.js.map